from typing import Union, List

from WeiboBot.util import *


class Weibo:
    def __init__(self):
        # region 基本信息
        self.visible = DictField()  # 微博的可见性及指定可见分组信息
        self.created_at = StrField()  # 微博创建时间
        self.id = StrField()  # 微博ID
        self.mid = StrField()  # 微博MID
        self.can_edit = BoolField()  # 是否可以编辑
        self.show_additional_indication = IntField()  # 是否显示额外信息
        self.text = StrField()  # 微博信息内容
        self.textLength = IntField()  # 微博信息内容字数
        self.source = StrField()  # 微博来源
        self.favorited = BoolField()  # 是否已收藏
        self.pic_ids = ListField()  # 微博的配图ID
        self.pic_types = StrField()  # 微博的配图类型
        self.pic_focus_point = ListField()  # 微博的配图中心点
        self.falls_pic_focus_point = ListField()  # 微博的配图中心点
        self.pic_rectangle_object = ListField()  # 微博的配图框
        self.pic_flag = IntField()  # 微博的配图标记
        self.thumbnail_pic = StrField()  # 微博的缩略图
        self.bmiddle_pic = StrField()  # 微博的中等尺寸图片
        self.original_pic = StrField()  # 微博的原始图片
        self.is_paid = BoolField()  # 是否付费
        self.mblog_vip_type = IntField()  # 微博的会员类型
        self.user = DictField()  # 微博作者的用户信息字段
        self.picStatus = StrField()  # 微博的配图状态
        self.reposts_count = IntField()  # 转发数
        self.comments_count = IntField()  # 评论数
        self.reprint_cmt_count = IntField()  # 转发评论数
        self.attitudes_count = IntField()  # 赞数
        self.pending_approval_count = IntField()  # 待审核数
        self.isLongText = BoolField()  #
        self.liked = BoolField()  # 是否已赞
        self.like_attitude_type = IntField()  # 点赞类型
        self.reward_exhibition_type = IntField()  # 打赏模块展示类型
        self.hide_flag = IntField()  # 隐藏类型
        self.mlevel = IntField()  # 微博等级
        self.darwin_tags = ListField()  # 微博的标签
        self.mblogtype = IntField()  # 微博类型
        self.more_info_type = IntField()  # 更多信息类型
        self.cardid = StrField()  # 卡片ID
        self.number_display_strategy = DictField()  #
        self.enable_comment_guide = BoolField()  #
        self.content_auth = IntField()  #
        self.pic_num = IntField()  #
        self.alchemy_params = DictField()  #
        self.reprint_type = IntField()  #
        self.can_reprint = BoolField()  #
        self.new_comment_style = IntField()  #
        self.page_info = DictField()  #
        self.pics = ListField()  #
        self.bid = StrField()  #
        self.status_title = StrField()  #
        self.ok = IntField()  #
        self.scheme = StrField()  #
        self.tipScheme = StrField()  #
        self.raw_text = StrField()  #
        self.title = DictField()  #
        self.repost_type = IntField()  #
        self.retweeted_status = DictField()  #
        self.edit_count = IntField()  #
        self.edit_at = StrField()  #
        self.version = IntField()  #
        self.gif_videos = ListField()  #
        self.reads = IntField()  # 阅读数
        self.rid = StrField()  #
        self.safe_tags = IntField()  #
        self.fid = IntField()  #
        self.pic_video = StrField()  #
        self.live_photo = ListField()  #
        self.pid = IntField()  #
        self.pidstr = StrField()  #
        self.jump_type = IntField()  #
        self.topic_id = StrField()  #
        self.sync_mblog = BoolField()  #
        self.is_imported_topic = BoolField()  #
        self.longText = DictField()
        self.mark = StrField()  #
        self.reward_scheme = StrField()
        self.state = IntField()  #
        self.expire_time = IntField()  #
        self.deleted = StrField()  #
        self.ad_state = IntField()  #
        self.verified_type_ext = IntField()
        self.verified_reason = StrField()
        self.mlevelSource = StrField()
        # endregion
        
        self.original_weibo: Union[Weibo, None] = None
        self.logger = get_logger(__name__)
    
    def parse(self, data):
        for k, v in data.items():
            if hasattr(self, k):
                setattr(self, k, v)
            else:
                self.logger.warning(f'{k} is not a valid attribute, type is {type(v)}')
        
        if self.retweeted_status != {}:
            self.original_weibo = Weibo()
            self.original_weibo.parse(self.retweeted_status)
    
    def detail_url(self) -> str:
        return f"https://m.weibo.cn/detail/{self.id}"
    
    def full_text(self) -> str:
        """
        未格式化的原文本
        :return:
        """
        if self.longText != {}:
            return self.longText['longTextContent']
        else:
            return self.text
    
    def weibo_id(self) -> int:
        return int(self.id)
    
    def user_uid(self) -> int:
        return int(self.user["id"])
    
    def video_url(self) -> str:
        url = ""
        if self.page_info.get("type", "") == "video" and "urls" in self.page_info:
            url = list(self.page_info["urls"].values())[0]
        return url
    
    def image_list(self) -> List[str]:
        return [img["large"]["url"] for img in self.pics]
    
    def thumbnail_image_list(self) -> List[str]:
        return [img["url"] for img in self.pics]  # 微博图片(缩略图)
    
    def is_visible(self) -> bool:
        return self.visible.get('type', 0) == 0



from enum import Enum

__all__ = ["VISIBLE", "MSG", "ACTION", "WEIBOERR", "WEIBO_WARNING"]


class VISIBLE(Enum):
    ALL = 0  # 全部
    ONLY_ME = 1  # 仅自己可见
    FRIENDS = 6  # 仅好友可见
    FOLLOWERS = 10  # 仅关注人可见


class MSG(Enum):
    NORMAL = 1  # 普通消息
    SUBSCRIPTION = 4  # 订阅消息


class ACTION(Enum):
    UNDONE = 0  # 未完成
    RUNNING = 1  # 运行中
    DONE = 2  # 已完成
    FAILED = 3  # 失败
    MAX_TRY = 4  # 超过最大尝试次数


class WEIBOERR(Enum):
    NO_DATA = 100011  # 没有数据


WEIBO_WARNING = [WEIBOERR.NO_DATA.value]



import asyncio
from typing import Union, List, Callable

from .action import Action
from .const import *
from .exception import *
from .message import Chat
from .net_tool import NetTool
from .user import User
from .util import *
from .weibo import Weibo


class Bot(User):
    def __init__(self, userName: str = "", password: str = "", cookies: str = "", loop_interval=5, action_interval=30):
        super(Bot, self).__init__()
        self.nettool = NetTool(userName, password, cookies)
        
        self.msg_handler: List[Callable] = []
        self.weibo_handler: List[Callable] = []
        self.weibo_read = set()
        
        self.action_list: list[Action] = []  # 待执行的动作列表
        self.action_interval = action_interval
        self.logger = get_logger(__name__)
        self.loop_interval = loop_interval
    
    async def login(self):
        login_result, self.id = await self.nettool.login()
        
        if login_result is True:
            self.logger.info(f"登录成功")
        else:
            raise LoginError("登录失败")
        
        await self.init_bot_info()
    
    async def init_bot_info(self):
        raw_data = await self.nettool.user_info(self.id)
        if raw_data["ok"] == 0:
            raise RequestError("获取用户信息失败")
        self.parse(raw_data["data"]["user"])
        
        self.logger.info(
            f"用户名:{self.screen_name},关注:{self.follow_count},粉丝:{self.followers_count},微博数量:{self.statuses_count}")
        self.logger.info(f"微博简介:{self.description}")
        self.logger.info(f"微博地址:{self.profile_url}")
        self.logger.info(f"微博头像:{self.profile_image_url}")
        self.logger.info(f"微博背景图:{self.cover_image_phone}")
    
    async def get_weibo(self, mid: Union[str, int]) -> Weibo:
        """
        获取微博实例
        
        :param mid:微博id
        :return: 微博实例
        """
        raw_data = await self.nettool.weibo_info(mid)
        oWeibo = Weibo()
        oWeibo.parse(raw_data)
        return oWeibo
    
    async def _post(self, content: str, visible: VISIBLE = VISIBLE.ALL) -> Weibo:
        """
        发布微博
        
        :param content:内容
        :param visible:可见性
        :return:新发出的微博
        """
        result = await self.nettool.post_weibo(content, visible)
        self.check_result(result)
        oWeibo = Weibo()
        oWeibo.parse(result["data"])
        self.logger.info(f"发送微博成功")
        return oWeibo
    
    def check_result(self, result: dict):
        if result["ok"] == 0:
            if result['errno'] in WEIBO_WARNING:
                self.logger.warning(f"错误类型{result['errno']},{result['msg']}")
            raise RequestError(f"错误类型{result['errno']},{result['msg']}")
    
    def post_action(self, content: str, visible: VISIBLE = VISIBLE.ALL):
        self.action_list.append(Action(self._post, content, visible))
    
    def repost_action(self, mid: Union[str, int], content: str = "转发微博", dualPost: bool = False):
        self.action_list.append(Action(self._repost, mid, content, dualPost))
    
    async def _repost(self, mid: Union[str, int], content: str = "转发微博", dualPost: bool = False) -> Weibo:
        """
        转发微博
        
        :param mid:微博id
        :param content:内容
        :param dualPost: 是否同时评论
        :return:新发出的微博
        """
        result = await self.nettool.repost_weibo(mid, content, dualPost)
        self.check_result(result)
        oWeibo = Weibo()
        oWeibo.parse(result["data"])
        self.logger.info(f"转发微博成功")
        return oWeibo
    
    async def send_chat(self, uid: Union[str, int], content: str):
        result = await self.nettool.send_chat(uid, content)
        self.check_result(result)
        oChat = Chat()
        oChat.parse(result["data"])
        return oChat
    
    async def chat_list(self, page: int = 1):
        result = await self.nettool.chat_list(page)
        self.check_result(result)
        return result["data"]
    
    async def chat_event(self):
        data = await self.chat_list()
        for dChat in data:
            unread = dChat["unread"]
            scheme = dChat["scheme"]
            if unread > 0 and scheme.find("gid=") == -1:
                oChat = await self.user_chat(dChat['user']["id"])
                oChat.msg_list = [oMsg for oMsg in oChat.msg_list[:unread] if oMsg.isDm()]
                for func in self.msg_handler:
                    await func(oChat)
    
    async def refresh_page(self):
        result = await self.nettool.refresh_page()
        self.check_result(result)
        return result["data"]
    
    async def weibo_event(self):
        try:
            result = await self.refresh_page()
        except RequestError as e:  # 刷新页面失败,可跳过此次刷新
            self.logger.warning(e)
            return
        for weibo in result["statuses"]:
            if weibo["id"] in self.weibo_read:
                continue
            self.weibo_read.add(weibo["id"])
            oWeibo = await self.get_weibo(weibo["id"])
            for func in self.weibo_handler:
                await func(oWeibo)
    
    async def user_chat(self, uid: Union[str, int], since_id: int = 0):
        result = await self.nettool.user_chat(uid, since_id)
        self.check_result(result)
        oChat = Chat()
        oChat.parse(result["data"])
        return oChat
    
    async def like(self, mid):
        result = await self.nettool.like(mid)
        self.check_result(result)
        return result["data"]
    
    def onNewMsg(self, func):
        if func not in self.msg_handler:
            self.msg_handler.append(func)
    
    def onNewWeibo(self, func):
        if func not in self.weibo_handler:
            self.weibo_handler.append(func)
    
    async def run_action(self):
        """
        执行所有的action
        如果成功或者超过最大尝试次数，则删除action
        
        :return:
        """
        for action in self.action_list:
            result, status = await action.run()
            if status == ACTION.MAX_TRY or status == ACTION.DONE:
                self.action_list.remove(action)
            
            await asyncio.sleep(self.action_interval)
    
    async def lifecycle(self):
        await asyncio.wait_for(self.login(), timeout=10)
        while True:
            await self.chat_event()
            await self.weibo_event()
            await self.run_action()
            await asyncio.sleep(self.loop_interval)
    
    def run(self):
        try:
            asyncio.get_event_loop().run_until_complete(self.lifecycle())
        except KeyboardInterrupt:
            pass



import traceback

from WeiboBot.const import ACTION
from WeiboBot.util import *


class Action:
    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.arg = args
        self.kwargs = kwargs
        self.status = ACTION.UNDONE
        self.run_time = 0
        self.logger = get_logger(__name__)
    
    async def run(self):
        if self.run_time > 5:
            self.status = ACTION.MAX_TRY
            return None, self.status
        self.status = ACTION.RUNNING
        self.run_time += 1
        try:
            result = await self.func(*self.arg, **self.kwargs)
        except Exception:
            self.status = ACTION.FAILED
            self.logger.error(traceback.format_exc())
            return None, self.status
        self.status = ACTION.DONE
        return result, self.status
    
    def __str__(self):
        return f'<Action: {self.func.__name__}>'



from typing import Dict

from .log import Log

__all__ = ["chat_header", "main_header", "IntField", "StrField", "BoolField", "DictField", "ListField", "get_logger"]


def main_header(cookies: bytes) -> Dict[str, str]:
    headers_raw = b'''
            accept: application/json, text/plain, */*
    accept-encoding: gzip, deflate, br
    accept-language: zh-CN,zh;q=0.9
    cookie: %b
    mweibo-pwa: 1
    referer: https://m.weibo.cn/
    sec-ch-ua: " Not A;Brand";v="99", "Chromium";v="96", "Google Chrome";v="96"
    sec-ch-ua-mobile: ?0
    sec-ch-ua-platform: "Windows"
    sec-fetch-dest: empty
    sec-fetch-mode: cors
    sec-fetch-site: same-origin
    user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36
    x-requested-with: XMLHttpRequest
    x-xsrf-token: 1d1b9c
            '''
    
    return formatHeader(headers_raw, cookies)


def chat_header(cookies: bytes) -> Dict[str, str]:
    headers_raw = b'''accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9
accept-encoding: gzip, deflate, br
accept-language: zh-CN,zh;q=0.9
cache-control: no-cache
cookie: %s
pragma: no-cache
sec-ch-ua: " Not A;Brand";v="99", "Chromium";v="99", "Google Chrome";v="99"
sec-ch-ua-mobile: ?0
sec-ch-ua-platform: "Windows"
sec-fetch-dest: empty
sec-fetch-mode: navigate
sec-fetch-site: same-origin
upgrade-insecure-requests: 1
user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.82 Safari/537.36'''
    
    return formatHeader(headers_raw, cookies)


def formatHeader(headers_raw: bytes, cookies: bytes) -> Dict[str, str]:
    """
    复制浏览器中的header
    """
    headers_raw = headers_raw % cookies
    headers = headers_raw.splitlines()
    headers_tuples = [header.split(b":", 1) for header in headers]
    
    result_dict = {}
    for header_item in headers_tuples:
        if not len(header_item) == 2:
            continue
        
        item_key: str = header_item[0].strip().decode("utf8")
        item_value: str = header_item[1].strip().decode("utf8")
        result_dict[item_key] = item_value
    
    return result_dict


def IntField() -> int:
    return 0


def StrField() -> str:
    return ""


def BoolField() -> bool:
    return False


def DictField() -> dict:
    return {}


def ListField() -> list:
    return []


def get_logger(name: str) -> Log:
    return Log(name)



from typing import Dict

from WeiboBot.util import *


class User:
    
    def __init__(self):
        self.id = IntField()  # 用户id
        self.screen_name = StrField()  # 用户昵称
        self.profile_image_url = StrField()  # 用户头像
        self.profile_url = StrField()  # 用户主页
        self.statuses_count = IntField()  # 微博数
        self.verified = BoolField()  # 是否是微博认证用户
        self.verified_type = IntField()  # 认证类型
        self.close_blue_v = BoolField()  # 是否关注微博蓝V
        self.description = BoolField()  # 用户描述
        self.gender = StrField()  # 性别
        self.mbtype = IntField()
        self.urank = IntField()
        self.mbrank = IntField()
        self.follow_me = IntField()  # 是否关注我
        self.following = IntField()  # 我是否关注
        self.follow_count = IntField()  # 关注数
        self.followers_count = StrField()  # 粉丝数
        self.followers_count_str = StrField()  # 粉丝数
        self.cover_image_phone = StrField()  # 主页头图
        self.avatar_hd = StrField()  # 高清头像
        self.like = BoolField()  # 是否喜欢
        self.like_me = BoolField()  # 是否喜欢我
        
        self.logger = get_logger(__name__)
    
    def parse(self, info: Dict):
        for k, v in info.items():
            if hasattr(self, k):
                setattr(self, k, v)
            else:
                self.logger.warning(f'{k} is not a valid attribute, type is {type(v)}')



import logging
import logging.handlers
import os
import sys
from logging import Logger


class Log(Logger):
    def __init__(self, name: str, level=logging.DEBUG, is_print=True, is_file=True):
        super(Log, self).__init__(name, level)
        if is_print:
            s_handler = logging.StreamHandler(sys.stdout)
            s_handler.setFormatter(
                logging.Formatter(f"%(asctime)s - %(levelname)s - {name}[%(funcName)s][:%(lineno)d] - %(message)s"))
            self.addHandler(s_handler)
        if is_file:
            log_path = f"{os.getcwd()}/Log/WeiboBot/"
            if not os.path.exists(log_path):
                os.makedirs(log_path)
            f_handler = logging.handlers.TimedRotatingFileHandler(log_path + f'/WeiboBot.log', encoding='utf8')
            f_handler.suffix = ".%Y%m%d_%H"
            f_handler.setFormatter(
                logging.Formatter(f"%(asctime)s - %(levelname)s - {name}[%(funcName)s][:%(lineno)d] - %(message)s"))
            self.addHandler(f_handler)



from typing import List, Dict

from WeiboBot.user import User
from WeiboBot.util import *
from .message import Message


class Chat:
    def __init__(self):
        self.following = BoolField()  # 是否关注
        self.last_read_mid = IntField()  #
        self.title = StrField()  # 标题
        self.total_number = IntField()  # 总数
        self.users = DictField()
        
        self.user_dict: Dict[int, User] = {}
        self.msg_list: List[Message] = []
        
        self.logger = get_logger(__name__)
    
    def parse(self, data):
        for k, v in data.items():
            if k == "msgs":
                continue
            if hasattr(self, k):
                setattr(self, k, v)
            else:
                self.logger.warning(f'{k} is not a valid attribute, type is {type(v)}')
        
        if data["msgs"]:
            for v in data["msgs"]:
                msg = Message()
                msg.parse(v)
                self.msg_list.append(msg)
        if data["users"]:
            for k, v in data["users"].items():
                user = User()
                user.parse(v)
                self.user_dict[int(k)] = user
    
    def since_id(self):
        if self.msg_list:
            return self.msg_list[0].id
        return 0



from WeiboBot.const import *
from WeiboBot.util import *


class Message:
    def __init__(self):
        self.created_at = StrField()  #
        self.dm_type = IntField()  #
        self.id = StrField()  #
        self.media_type = IntField()  #
        self.msg_status = IntField()  #
        self.recipient_id = StrField()  # 收件人id
        self.recipient_screen_name = StrField()  # 收件人的昵称
        self.sender_id = StrField()  # 发送者id
        self.sender_screen_name = StrField()  # 发送者的昵称
        self.text = StrField()  # 内容
        self.attachment = DictField()  # 附件
        
        self.logger = get_logger(__name__)
    
    def parse(self, data):
        for k, v in data.items():
            if hasattr(self, k):
                setattr(self, k, v)
            else:
                self.logger.warning(f'{k} is not a valid attribute, type is {type(v)}')
    
    def isDm(self):
        """
        判断是否是私信
        :return:
        """
        return self.dm_type == MSG.NORMAL.value
    
    def isSubscription(self):
        """
        判断是否是订阅
        :return:
        """
        return self.dm_type == MSG.SUBSCRIPTION.value



import setuptools

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="WeiboBot",
    version="0.0.5",
    author="Merlin",
    author_email="merlin@merlinblog.cn",
    description="基于微博H5 API开发的机器人框架",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MerlinCN/WeiboBot",
    packages=setuptools.find_packages(),
    install_requires=["requests"],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



import json
import re
import time
import traceback
from typing import Dict, Tuple, Union

import requests

from .const import *
from .exception import *
from .util import *


class NetTool:
    def __init__(self, userName: str = "", password: str = "", cookies: str = ""):
        super(NetTool, self).__init__()
        # 暂不支持用户名和密码登录 v1.0
        if userName and password:
            raise NotImplementedError("暂不支持用户名和密码登录")
        elif cookies:
            # 如果有cookies则直接使用cookies登录
            pass
        else:
            raise LoginError("登录失败!请输入cookies")
        
        self.cookies: str = cookies
        self.header: Dict[str, str] = main_header(bytes(self.cookies, encoding="utf-8"))
        self.mainSession: requests.session = requests.session()
        self.chat_header = chat_header(bytes(self.cookies, encoding="utf-8"))
        
        self.st_times = 0  # 获取st的次数
        self.logger = get_logger(__name__)
    
    def add_ref(self, value: str) -> Dict[str, str]:
        self.header["referer"] = value
        return self.header
    
    async def get(self, url: str, params: Dict = None, header=None) -> Dict:
        if header is None:
            header = self.header
        r = self.mainSession.get(url, headers=header, params=params)
        if r.status_code != 200:
            raise RequestError(f"网络错误!状态码:{r.status_code}\n{r.text}")
        try:
            result = r.json()
            return result
        except Exception:
            raise RequestError(traceback.format_exc())
    
    async def post(self, url: str, params: Dict = None) -> Dict:
        r = self.mainSession.post(url, headers=self.header, data=params)
        if r.status_code != 200:
            raise RequestError(f"网络错误!状态码:{r.status_code}\n{r.text}")
        try:
            result = r.json()
            return result
        except Exception:
            raise RequestError(traceback.format_exc())
    
    async def login(self) -> Tuple[bool, int]:
        url = "https://m.weibo.cn/api/config"
        self.add_ref(url)
        data = await self.get(url)
        isLogin = data['data']['login']
        if not isLogin:
            return False, 0
        st = data["data"]["st"]
        self.header["x-xsrf-token"] = st
        roleid = int(data['data']['uid'])
        return True, roleid
    
    async def st(self):
        """
        获得session token

        :return: session token
        """
        try:
            data = await self.get("https://m.weibo.cn/api/config")
        except RequestError:
            if self.st_times > 5:
                self.logger.error("获取st失败!")
                st = self.header["x-xsrf-token"]
                return st
            self.st_times += 1
            time.sleep(0.5)
            return await self.st()
        st = data["data"]["st"]
        return st
    
    async def user_info(self, user_id: int):
        url = f"https://m.weibo.cn/profile/info?uid={user_id}"
        self.add_ref(f"https://m.weibo.cn/profile/{user_id}")
        return await self.get(url)
    
    async def post_weibo(self, content: str, visible: VISIBLE):
        params = {
            "content": content,
            "visible": visible.value,
            "st": await self.st(),
            "_spr": "screen:2560x1440"
        }
        
        return await self.post("https://m.weibo.cn/api/statuses/update", params=params)
    
    async def repost_weibo(self, mid: Union[str, int], content: str, dualPost: bool):
        self.add_ref(f"https://m.weibo.cn/compose/repost?id={mid}")
        self.header["x-xsrf-token"] = await self.st()
        data = {
            "id": mid,
            "content": content,
            "mid": mid,
            "st": self.header["x-xsrf-token"],
            "_spr": "screen:2560x1440",
            "dualPost": int(dualPost)
        }
        return await self.post("https://m.weibo.cn/api/statuses/repost", params=data)
    
    async def weibo_info(self, mid: Union[str, int]) -> dict:
        r = self.mainSession.get(f"https://m.weibo.cn/detail/{mid}", headers=self.header)
        try:
            return json.loads(re.findall(r'(?<=render_data = \[)[\s\S]*(?=\]\[0\])', r.text)[0])["status"]
        except Exception:
            raise RequestError(traceback.format_exc())
    
    async def send_chat(self, uid: Union[str, int], content: str):
        params = {
            "uid": int(uid),
            "content": content,
            "st": await self.st(),
            "_spr": "screen:2560x1440"
        }
        return await self.post("https://m.weibo.cn/api/chat/send", params=params)
    
    async def user_chat(self, uid: Union[str, int], since_id: int):
        params = {"count": 20, "uid": uid, "since_id": since_id}
        return await self.get("https://m.weibo.cn/api/chat/list", params=params)
    
    async def chat_list(self, page: int):
        params = {"page": page}
        return await self.get("https://m.weibo.cn/message/msglist", params=params)
    
    async def refresh_page(self):
        self.add_ref("https://m.weibo.cn/")
        return await self.get("https://m.weibo.cn/feed/friends?")
    
    async def like(self, mid):
        self.add_ref("https://m.weibo.cn/")
        params = {
            "id": mid,
            "attitude": "heart",
            "st": await self.st(),
            "_spr": "screen:2560x1440"
        }
        return await self.post("https://m.weibo.cn/api/attitudes/create", params=params)
